'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var PublicKey = require('./PublicKey.cjs');
var arrays = require('./utils/arrays.cjs');

const signTransaction = async (transaction, signers) => signers.reduce(async (promise, signer) => {
  const unsigned = await promise;
  return signer.signTransaction(unsigned);
}, Promise.resolve(transaction));
const signAllTransactions = async transactionsWithSigners => {
  const transactions = transactionsWithSigners.map(item => item.transaction);
  const signersWithTransactions = transactionsWithSigners.reduce((all, {
    signers
  }, index) => {
    signers.forEach(signer => {
      const item = all.find(item => PublicKey.samePublicKey(item.signer.publicKey, signer.publicKey));
      if (item) {
        item.indices.push(index);
      } else {
        all.push({
          signer,
          indices: [index]
        });
      }
    });
    return all;
  }, []);
  return signersWithTransactions.reduce(async (promise, {
    signer,
    indices
  }) => {
    const transactions = await promise;
    if (indices.length === 1) {
      const unsigned = transactions[indices[0]];
      transactions[indices[0]] = await signer.signTransaction(unsigned);
      return transactions;
    }
    const unsigned = indices.map(index => transactions[index]);
    const signed = await signer.signAllTransactions(unsigned);
    indices.forEach((index, position) => {
      transactions[index] = signed[position];
    });
    return transactions;
  }, Promise.resolve(transactions));
};
const isSigner = value => 'publicKey' in value;
const uniqueSigners = signers => arrays.uniqueBy(signers, PublicKey.samePublicKey);
const createNoopSigner = publicKey => ({
  publicKey,
  async signMessage(message) {
    return message;
  },
  async signTransaction(transaction) {
    return transaction;
  },
  async signAllTransactions(transactions) {
    return transactions;
  }
});
const createNullSigner = () => new NullSigner();
class NullSigner {
  // TODO(loris): Custom errors.
  get publicKey() {
    throw new Error('Method not implemented.');
  }
  signMessage() {
    throw new Error('Method not implemented.');
  }
  signTransaction() {
    throw new Error('Method not implemented.');
  }
  signAllTransactions() {
    throw new Error('Method not implemented.');
  }
}

exports.NullSigner = NullSigner;
exports.createNoopSigner = createNoopSigner;
exports.createNullSigner = createNullSigner;
exports.isSigner = isSigner;
exports.signAllTransactions = signAllTransactions;
exports.signTransaction = signTransaction;
exports.uniqueSigners = uniqueSigners;
//# sourceMappingURL=Signer.cjs.map
